Program NMPef_old
! Three dimensional, non-stationary, multi-scale PEF-based interpolator
! niter - number of iterations to find NSMSPEF
! rad - radial or laplacian smoothing of micropatches (not yet implemented)
! eps - value of epsilon for NSMSPEF regularization
! filliter - number of iterations to fill data with NSMSPEF
! maskin - mask of known data (ones where data is known, zeroes where data is missing)
! a - size of PEF, center - position of leading 1 in PEF, gap - size of zeroes after 1
! patch - number of patches in each dimension (should divide cleanly in all dimensions)
! sc - scales to use, based upon the first dimension.  Must include original scale

use sep
use nhelix
use createnhelixmod
use nmisinput
use compress
use print
use nmpefe
use pef
use io_nfilt


  implicit none

  integer, external                      :: getch
  integer, dimension (:)  , pointer      :: sc, a, ndat, patch, fakepatch,tmppch
  integer                                :: tt(1000), niter, np, nsc, i, nb, j
  real,    dimension (:)  , pointer      :: data, k0, killdata, dummask
  
  integer, dimension (:)  , allocatable  :: center, gap, nn, cartflt, pos, psize, linflt, nt
  integer, dimension (:,:), allocatable  :: nnbank
  real,    dimension (:)  , allocatable  :: dd, oo, rbounds
  logical, dimension (:)  , allocatable  :: lbounds
  real,    dimension (:)  , pointer      :: ddat, odat, wei
  real                                   :: d1, d2, d3, o1, o2, o3, eps
  logical                                :: rad, ritestat, stat
  type(nfilter)                          :: aa
  character(len=1024)                    :: out

  call sep_init(SOURCE)
  call from_param("dim",dim)
!  dim = sep_dimension()

  write(0,*)"dimension",dim
  
  allocate(nn(dim), dd(dim), oo(dim), a(dim), center(dim), gap(dim), patch(dim))
  write(0,*)"NMPef 1"
! Read history of data
  call from_history( "m", nn)
  call from_history( "d", dd)
  call from_history( "o", oo)
  call from_history( "n1", nb)
  write(0,*)"NMPef 2"

! Read parameters
  call from_param ("niter", niter, 50)
  call from_param ("rad",rad,.false.)
  call from_param ("eps",eps,0.1)
  call from_param ("out",out,'npeffy.H')
  write(0,*)"NMPef 3"

! Read filter parameters
  call from_param( "a", a )
  center=1;  where( a(2:) > 1) center (:dim-1) = a (:dim-1)/2+1
  call from_param ("center", center, center)
  gap = 0
  call from_param ("gap", gap, gap)
  patch=1
  call from_param ("patch", patch, patch)
  np = product(patch) ! total number of micropatches
  write(0,*)"NMPef 4"

! Read scale parameters
  call from_param ("nsc", nsc)
  write(0,*)"NMPef 4.1"
  allocate( nt(nsc*dim), nnbank(nsc,dim))
  write(0,*)"NMPef 4.2"
  call from_history ("nbank", nt)
  write(0,*)"NMPef 4.3"
  nnbank=reshape(nt,(/nsc,dim/))
  write(0,*)"NMPef 5"
  
! allocate data arrays
  allocate( data(nb), k0(nb), rbounds(nb))
  write(0,*)"NMPef 6"

! Read databank and maskbank


  call sep_read( data )
  k0 = 1.
  if (exist_file ("maskin"))  call sep_read( k0, "maskin")
  if (exist_file ("bounds"))  call sep_read( rbounds, "bounds")
  
  write(0,*)"NMPef 7"

  allocate(lbounds(nb))
  lbounds=.false.
  where (rbounds<>0) lbounds=.true.
  deallocate(rbounds)

! allocate helix
write(0,*)"before createhelix n", nn
write(0,*)"center", center
write(0,*)"gap", gap
write(0,*)"a", a
write(0,*)"np", np
write(0,*)"size rbounds", size(rbounds)
! create patch table for filter
  allocate(psize(dim), pos(dim), cartflt(dim), linflt(product(nn)))

write(0,*)"after allocate"
! check to see if the patch size cleanly divides the data space
  do i=1,dim
     if (mod(nn(i),patch(i))>0) then
        write(0,*)"patch choice poor",mod(patch(i),ndat(i))
     end if
  end do

  psize = nn/patch
write(0,*)"after patch check",nn,size(pos),size(patch),size(cartflt),size(linflt)
  do i = 1,product(nn)
     call line2cart(nn,i,pos)
     cartflt = ((pos-1)/psize) +1
     call cart2line(patch,cartflt,linflt(i))
  end do

write(0,*)"After linflt",nn,size(linflt),minval(linflt),maxval(linflt)
  aa = createnhelix( nn, center, gap, a, linflt)
write(0,*)"after createhelix"
! bound filter (removed)
write(0,*)"before main"
! call main program
  call nmpef_find( data, aa, nnbank, dd, oo, a, niter, lbounds, k0, patch, eps )
write(0,*)"after main lbounds",size(lbounds)
write(0,*)"after main aamis", size(aa%mis)

! reset bounds on filter CHANGE TO BOUNDBANK
!  aa%mis=lbounds(1:nb)

write(0,*)"after mis set" 

! write out filter
  ritestat=rite_nfilt(out,aa,nn,oo,dd)
  write(0,*)"filter written?",ritestat

  call sep_close ()
  write(0,*)"end1"
  deallocate( data, k0, lbounds)
  write(0,*)"end2"
  call ndeallocate(aa)
  write(0,*)"end3"
  deallocate (nn, dd, oo, center, gap, a, patch, psize, pos, cartflt, linflt, nnbank)
  write(0,*)"end4"
  call exit (0)
end program NMPef_old

































































