Program NSFill
! Three dimensional, non-stationary, multi-scale PEF-based interpolator
! niter - number of iterations to find NSMSPEF
! rad - radial or laplacian smoothing of micropatches (not yet implemented)
! eps - value of epsilon for NSMSPEF regularization
! filliter - number of iterations to fill data with NSMSPEF
! maskin - mask of known data (ones where data is known, zeroes where data is missing)
! a - size of PEF, center - position of leading 1 in PEF, gap - size of zeroes after 1
! patch - number of patches in each dimension (should divide cleanly in all dimensions)
! sc - scales to use, based upon the first dimension.  Must include original scale

use sep
use nhelix
use createnhelixmod
use nbound_bob_mod
use pef
use io_nfilt
use nsfill

  implicit none
  integer                              :: n1, n2, n3, a1, a2, a3, niter, nsc, na, i, linflt, dim
  integer, external                    :: getch
  integer, dimension (:), pointer      :: sc, a, ndat, patch, fakepatch,tmppch
  integer                              :: tt(1000)
  real, dimension (:), pointer         :: data, k0, killdata, dummask
  integer, dimension (:), allocatable  :: center, gap, psize, pos, cartflt
  real, dimension (:), pointer         :: ddat, odat, wei
  real                                 :: d1, d2, d3, o1, o2, o3, eps
  logical                              :: rad, readstat, stat, lmask(:)
  type(nfilter)                        :: aa
  character(len=1024)                  :: filt

  call sep_init(SOURCE)
  dim = sep_dimension()
! Read history of data
  call from_history( "n1", n1);       call from_history( "n2", n2);       call from_history( "n3", n3)
  call from_history( "d1", d1, 1.0);  call from_history( "d2", d2, 1.0);  call from_history( "d3", d3, 1.0)
  call from_history( "o1", o1, 0.0);  call from_history( "o2", o2, 0.0);  call from_history( "o3", o3, 0.0)
  call to_history( "n", (/n1,n2,n3/) )
  call from_param ("filt",filt,'filt.H')
  call from_param ("niter",niter,100)


  allocate(ndat(dim), a(dim), ddat(dim), odat(dim), center(dim), gap(dim), patch(dim), fakepatch(dim) )
  ndat = (/n1,n2,n3/);
  allocate( data(product(ndat)), k0(product(ndat)), tmppch(product(ndat)), wei(product(ndat)) )
  call sep_read( data )
  call sep_read( k0, "maskin")
  readstat = read_nfilt(filt,aa)
! allocate filter
  allocate(lmask(size(k0)))
  lmask=.false.
  where(k0/=0) lmask=.true.
  wei = 1.
  call nbound_bob( ndat, aa)
  call nsfill_data(niter, data, aa, lmask, .false., wei )
  call sep_write (data)
  call sep_close ()
  deallocate( data)
  deallocate( k0)
  call ndeallocate(aa)
  deallocate (ndat) 
  call exit (0)
end program NSFill
