module coarsen_new
! Coarsen mesh.
  use adj_mod
  use cartesian
  use lint
  use error_check

  implicit none
  real,   dimension( :, :), pointer :: xyp 
  real,    dimension(:), allocatable :: loc_fine
  integer, dimension(:), allocatable :: iloc_fine
  contains
  subroutine coarsen_init ( fn,fo,fd,cn,co,cd)
    integer               ::                  in, i, id, ie, dim, nd
    real,    dimension(:) ::   fo,fd,   co,cd
    integer, dimension(:) ::fn,      cn
    logical, dimension(:) ::                  mask

    nd=product(fn)
    dim = size(fo)
    allocate (xyp(nd,dim))
    allocate (loc_fine(dim),iloc_fine(dim))
   !loop through the fine space
    do i=1,nd
       call line2cart(fn,i,iloc_fine) ! my current index in fine space
       loc_fine= (iloc_fine-1.)*fd+fo ! my current position in fine space
       xyp(i,:) = loc_fine
    end do
!call rmms2(xyp,"xyp  ")
    call lint_init( cn, co, cd, xyp)
    deallocate(loc_fine,iloc_fine)
  end subroutine coarsen_init

  function coarsen_lop ( adj, add, mm, dd) result(stat)
    integer            :: stat 
    logical,intent(in) :: adj,add 
    real,dimension(:)  :: mm,dd 
    call adjnull (adj,add,mm,dd )
    call coarsen_lop2(adj,mm,dd )
    stat=0

  end function coarsen_lop

  subroutine coarsen_lop2(adj,mm,dd)
    logical,intent(in) :: adj
    real, dimension(:) ::     mm,dd
    integer stat1
    stat1 = lint_lop( adj, .true., mm, dd)
  end subroutine coarsen_lop2

  subroutine  coarsen_close()
    deallocate (xyp)
    call lint_close()
  end subroutine coarsen_close
end module coarsen_new














