module coarsen_norm

use sep
use adj_mod
use chain_mod
use coarsen_new
use weight

implicit none

real, dimension(:), pointer :: fold

contains

! ---------------------------------------------------------------------------
  subroutine coarsen_norm_init( fo,  fd,n,  co,   cd,m,maskin,normlz )
    optional      ::                                   maskin,normlz
    integer       :: stat,              n(:),        m(:)
    real          ::            fo(:),fd(:),co(:),cd(:)
    real, pointer ::                                   maskin(:),   mask(:) ! known mask on fine grid
    logical       :: norm,                                    normlz      ! Normalize or not?
    
    ! Handle the optional arguments.
    norm = .true.
    if( present(normlz) ) norm=normlz
    if( .not. present(maskin) ) then
       allocate( mask(product(n)) )
       mask=1.0                                     ! data space vector of ones
    else
       mask => maskin
    end if
    
    ! determine fold by refining mask
    allocate( fold(product(m)) )
    if( norm) then
       call coarsen_init( fo, fd, n, co, cd, m)
       stat = coarsen_lop( .true., .false., fold, mask )
       call coarsen_close()     
    else
       fold = 1.0
    end if
    where( fold>0 ) fold=1./fold
    call weight_init(fold)       
    if( .not. present(maskin) )  deallocate( mask )
    call coarsen_init( fo, fd, n, co, cd, m)
!                                     write(0,*)"min/max/size fold",minval(fold),maxval(fold),size(fold)
  end subroutine coarsen_norm_init
! ---------------------------------------------------------------------------
        
! ---------------------------------------------------------------------------
        
  function coarsen_norm_lop ( adj, add, mm, dd) result(stat)
    integer            :: stat 
    logical,intent(in) :: adj,add 
    real,dimension(:)  :: mm,dd 
    call adjnull (adj,add,mm,dd )
    call coarsen_norm_lop2(adj,add,mm,dd )
    stat=0
  end function 

  subroutine coarsen_norm_lop2( adj,add, mm, dd)
    logical            :: adj,add
    real, dimension(:) :: mm,dd,temp 

    allocate( temp(size(mm)) )

! chain refine with weight for normalization
    call chain( coarsen_lop, weight_lop, adj,add, mm, dd, temp )
    deallocate(temp)
    
  end subroutine coarsen_norm_lop2
! ---------------------------------------------------------------------------

! ---------------------------------------------------------------------------
  subroutine coarsen_norm_close( )
    deallocate( fold)
    call coarsen_close()
  end subroutine coarsen_norm_close
! ---------------------------------------------------------------------------

! ---------------------------------------------------------------------------
  subroutine get_fold( foldout )
    real, dimension(:), pointer :: foldout
    foldout = fold
  end subroutine get_fold
! ---------------------------------------------------------------------------


end module coarsen_norm

