module coarsen_norm_new

use sep
use adj_mod
use chain_mod
use coarsen_new
use weight
use error_check

implicit none

real, dimension(:), pointer :: fold

contains

! ---------------------------------------------------------------------------
  subroutine coarsen_norm_init( n,fo,   fd, m,co,   cd  ,maskin,normlz )
    optional      ::                                   maskin,normlz
    integer       :: stat,      n(:),       m(:)
    real          ::              fo(:),fd(:),co(:),cd(:)
    real, pointer ::                                     maskin(:),   mask(:) ! known mask on fine grid
    logical       :: norm,                                      normlz      ! Normalize or not?
    
    ! Handle the optional arguments.
    norm = .true.
    if( present(normlz) ) norm=normlz
    if( .not. present(maskin) ) then
       allocate( mask(product(n)) )
       mask=1.0                                     ! data space vector of ones
    else
       mask => maskin
    end if
    
    ! determine fold by refining mask
    allocate( fold(product(m)) )
    if( norm) then
       call coarsen_init( n,fo, fd, m, co, cd)
write(0,*) "fn cn",n,m
write(0,*) "fo co",fo,co
write(0,*) "fd cd",fd,cd

write(0,*) "coarsen_norm mask",count(mask>0),size(mask)
       stat = coarsen_lop( .true., .false., fold, mask )
       call coarsen_close()
write(0,*) "coarsen_norm fold",count(fold>0),size(fold)
    else
       fold = 1.0
    end if
    where( fold>0 ) fold=1./fold
    call weight_init(fold)
    if( .not. present(maskin) )  deallocate( mask )
    call coarsen_init( n,fo, fd, m, co, cd)
                                     write(0,*)"min/max/size fold",minval(fold),maxval(fold),size(fold)
  end subroutine coarsen_norm_init
! ---------------------------------------------------------------------------
        
! ---------------------------------------------------------------------------
        
  function coarsen_norm_lop ( adj, add, mm, dd) result(stat)
    integer            :: stat 
    logical,intent(in) :: adj,add 
    real,dimension(:)  :: mm,dd 
    call adjnull (adj,add,mm,dd )
    call coarsen_norm_lop2(adj,add,mm,dd )
    stat=0
  end function 

  subroutine coarsen_norm_lop2( adj,add, mm, dd)
    logical            :: adj,add
    real, dimension(:) :: mm,dd,temp 

    allocate( temp(size(mm)) )
! chain refine with weight for normalization
write(0,*)"b mix/max/size mm",minval(mm),maxval(mm),size(mm)
write(0,*)"b mix/max/size dd",minval(dd),maxval(dd),size(dd)
    call chain( coarsen_lop, weight_lop, adj,add, mm, dd, temp )
write(0,*)"a mix/max/size mm",minval(mm),maxval(mm),size(mm)
write(0,*)"a mix/max/size dd",minval(dd),maxval(dd),size(dd)
    deallocate(temp)
  end subroutine coarsen_norm_lop2
! ---------------------------------------------------------------------------

! ---------------------------------------------------------------------------
  subroutine coarsen_norm_close( )
    deallocate( fold)
    call coarsen_close()
  end subroutine coarsen_norm_close
! ---------------------------------------------------------------------------

! ---------------------------------------------------------------------------
  subroutine get_fold( foldout )
    real, dimension(:), pointer :: foldout
    foldout = fold
  end subroutine get_fold
! ---------------------------------------------------------------------------


end module coarsen_norm_new

