module fill_ns_old

  use lint
  use nhelix
  use nhelicon
  use npolydiv
  use cartesian
  use nbound_bob_mod
  use cgstep_mod
  use random
  use solver_prc_mod
  use solver_reg_mod
  use solver_smp_mod
  use error_check
  use rescale_bill
  use mask1
  
  implicit none

contains

  subroutine fill          ( pef,points,coord,model,n,o,d,prec,sigma,feps,fiter)
    type(nfilter)         :: pef
    real,dimension(:)     ::     points,      model,  o,d
    real,dimension(:,:),pointer   ::    coord
    integer, dimension(:) ::                        n
    logical               ::                              prec
    real                  ::                                    sigma,feps
    integer               ::                                              fiter,stat,i
    real,dimension(:),allocatable:: rand,drand
    
write(0,*)"fill start"
    allocate(rand(product(n)))
    call nbound_bob(n,pef)
write(0,*)"fill after bound pef"
call pefcheck(pef)
    do i=1,size(rand)
       rand(i)=random_normal()*sigma
    end do
write(0,*)"fill after rand"
    call lint_init(n,o,d,coord)
write(0,*)"fill after lint init"
    call rmms(points,"point")
    if (prec) then
       call npolydiv_init(size(model),pef)
       call solver_prc(m=model,                                    &
                       d=points,                                   &
                       Fop=lint_lop,                               &
                       Sop=npolydiv_lop,                           &
                       stepper=cgstep,                             &
                       nSop=(product(n)),                          &
                       niter=fiter,                                &
                       rm0=rand,                                   &
                       eps=feps,                                   &
                       verb=.true.)
       call npolydiv_close()

    else

       call rmms(rand,"rand ")
       call nhelicon_init(pef)
       call solver_reg(m=model,                                    &
                       d=points,                                   &
                       Fop=lint_lop,                               &
                       Aop=nhelicon_lop,                           &
                       stepper=cgstep,                             &
                       nAop=(product(n)),                          &
                       niter=fiter,                                &
                       rm0=rand,                                   &
                       eps=feps,                                    &
                       verb=.true.)
       call nhelicon_close()

    end if

    call lint_close()
    call cgstep_close()
    deallocate(rand)
  end subroutine fill

  subroutine fill_grid         (pef,datagrid,mask,model,prec,sigma,eps,iter)
    type(nfilter)            :: pef
    real,dimension(:),pointer::     datagrid,     model
    logical,dimension(:),pointer::           mask
    integer                  ::                                   iter, i
    real                     ::                              sigma,eps
    logical                  ::                         prec
    real,dimension(:),allocatable :: rand

write(0,*)"begin fill_grid"
    allocate(rand(size(datagrid)))
    do i=1,size(rand)
       rand(i)=random_normal()*sigma
    end do
write(0,*)"fill_grid 1"
    
    call mask1_init(mask)
write(0,*)"fill_grid 2"
    
    if (prec) then
       call npolydiv_init(size(model),pef)
       call solver_prc(m=model,          &
                       d=datagrid,       &
                       Fop=mask1_lop,    &
                       Sop=npolydiv_lop, &
                       stepper=cgstep,   &
                       nSop=size(model), &
                       niter=iter,       &
                       rm0=rand,         &
                       eps=0.,           &
                       Wop=mask1_lop,    &
                       verb=.true.)
       call npolydiv_close()
    else
       call nhelicon_init(pef)


call rmms(model,"model")
call rmms(datagrid,"datag")
call lmms(mask,"mask ")
call rmms(rand,"rand ")
write(0,*)"iter",iter
       call solver_reg(m=model,          &
                       d=datagrid,       &
                       Fop=mask1_lop,    &
                       Aop=nhelicon_lop, &
                       stepper=cgstep,   &
                       nAop=size(model), &
                       niter=iter,       &
                       rm0=rand,         &
                       eps=eps,          &
                       Wop=mask1_lop,    &
                       verb=.true.)
       call nhelicon_close()
    end if
    deallocate(rand)
  end subroutine fill_grid

end module fill_ns
