module lint
! Linear interpolation in 1D to 5D
  use adj_mod
  use error_check
  implicit none
  integer          :: n(:), nd, dim
  real             :: o(:), d(:), f(:), g(:)
  real, pointer    :: xy(:,:)

  contains
  subroutine lint_init ( n_in,o_in,   d_in,xy_in )
    integer ::           n_in(:) 
    real    ::                o_in(:),d_in(:) 
    real, pointer ::                       xy_in(:,:) 
    dim= size(n_in)
    nd = size(xy_in,1)
    allocate(n(dim),o(dim),d(dim),f(dim),g(dim))
    n = n_in 
    o = o_in 
    d = d_in 
    xy => xy_in
  end subroutine lint_init

  function lint_lop     ( adj,add, mm,   dd) result(stat)
    integer            ::                         stat 
    logical,intent(in) :: adj,add 
    real               ::          mm(:),dd(:) 
    call adjnull (adj,add,mm,dd )
!write(0,*)"before"
!call rmms(mm,"mm   ")
!call rmms(dd,"dd   ")
!call rmms2(xy,"xy   ")
    if (dim==1) call lint1_lop2(adj,mm,dd )
    if (dim==2) call lint2_lop2(adj,mm,dd )
    if (dim==3) call lint3_lop2(adj,mm,dd )
    if (dim==4) call lint4_lop2(adj,mm,dd )
    if (dim==5) call lint5_lop2(adj,mm,dd )
!write(0,*)"after"
!call rmms(mm,"mm   ")
!call rmms(dd,"dd   ")
    stat=0
  end function lint_lop

  subroutine lint1_lop2(adj,mm,dd)
    integer :: i,im,id
    real    :: f,fx,gx
    logical ::          adj
    real,dimension(:) ::    mm,dd
    do id= 1, size(dd)
       f = (xy(id)-o(1))/d(1)
       i=f
       im= 1+i
       if (f==real(size(mm))) then
          if (adj) then
             mm(im)=mm(im)+dd(id)
          else
             dd(id)=dd(id)+mm(im)
          end if
       end if
       if ( 1<=im .and. im< size(mm)) then
          fx=f-i
          gx= 1.-fx
          if ( adj) then
             mm(im  ) =                        mm(im  ) + gx*dd(id)
             mm(im+1) =                        mm(im+1) +           fx*dd(id)
          else
             dd(id)   =                        dd(id)   + gx*mm(im)+fx*mm(im+1)
          end if
       else
!          write(0,*)"oh, shit!"
       end if
    end do
  end subroutine lint1_lop2

  subroutine lint2_lop2(adj,mm,dd)
    logical,intent(in) :: adj  
    real, dimension (n(1),n(2))  :: mm 
    real, dimension (:)  :: dd
    integer :: id, im, ix, iy, i(dim)
    real    :: fract
    do id= 1,nd
      f = (xy(id,:)-o)/d
      i=f
      ix= 1+i(1)
      iy= 1+i(2)
      if ( 1>ix .or. ix>=n(1)) then
!        write(0,*)"oh, shit!"
        cycle
      end if
      if ( 1>iy .or. iy>=n(2)) then
!        write(0,*)"oh, shit!"
        cycle
      end if
      f=f-i
      g= 1.-f
      if ( adj) then
        mm(ix  ,iy  ) = mm(ix  ,iy  ) + g(1)*g(2)*dd(id)
        mm(ix+1,iy  ) = mm(ix+1,iy  ) + f(1)*g(2)*dd(id)
        mm(ix  ,iy+1) = mm(ix  ,iy+1) + g(1)*f(2)*dd(id)
        mm(ix+1,iy+1) = mm(ix+1,iy+1) + f(1)*f(2)*dd(id)

      else
        dd(id) = dd(id) + g(1)*g(2)*mm(ix  ,iy  ) + f(1)*g(2)*mm(ix+1,iy  ) &
          &             + g(1)*f(2)*mm(ix  ,iy+1) + f(1)*f(2)*mm(ix+1,iy+1) &
          &             + g(1)*g(2)*mm(ix  ,iy  ) + f(1)*g(2)*mm(ix+1,iy  ) &
          &             + g(1)*f(2)*mm(ix  ,iy+1) + f(1)*f(2)*mm(ix+1,iy+1)
      end if
    end do
  end subroutine lint2_lop2

  subroutine lint3_lop2(adj,mm,dd)
    logical,intent(in) :: adj  
    real, dimension (n(1),n(2),n(3))  :: mm 
    real, dimension (:)  :: dd
    integer :: id, im, ix, iy, iz, i(dim)
    real    :: fract
    do id= 1,nd
      f = (xy(id,:)-o)/d
      i=f
      ix= 1+i(1)
      iy= 1+i(2)
      iz= 1+i(3)
      if ( 1>ix .or. ix>=n(1)) then
        cycle
      end if
      if ( 1>iy .or. iy>=n(2)) then
        cycle
      end if
      if ( 1>iz .or. iz>=n(3)) then
        cycle
      end if
      f=f-i
      g= 1.-f
      if ( adj) then
        mm(ix  ,iy  ,iz  ) = mm(ix  ,iy  ,iz  ) + g(1)*g(2)*g(3)*dd(id)
        mm(ix+1,iy  ,iz  ) = mm(ix+1,iy  ,iz  ) + f(1)*g(2)*g(3)*dd(id)
        mm(ix  ,iy+1,iz  ) = mm(ix  ,iy+1,iz  ) + g(1)*f(2)*g(3)*dd(id)
        mm(ix+1,iy+1,iz  ) = mm(ix+1,iy+1,iz  ) + f(1)*f(2)*g(3)*dd(id)
        mm(ix  ,iy  ,iz+1) = mm(ix  ,iy  ,iz+1) + g(1)*g(2)*f(3)*dd(id)
        mm(ix+1,iy  ,iz+1) = mm(ix+1,iy  ,iz+1) + f(1)*g(2)*f(3)*dd(id)
        mm(ix  ,iy+1,iz+1) = mm(ix  ,iy+1,iz+1) + g(1)*f(2)*f(3)*dd(id)
        mm(ix+1,iy+1,iz+1) = mm(ix+1,iy+1,iz+1) + f(1)*f(2)*f(3)*dd(id)

      else
        dd(id) = dd(id) + g(1)*g(2)*g(3)*mm(ix  ,iy  ,iz  ) + f(1)*g(2)*g(3)*mm(ix+1,iy  ,iz  ) &
          &             + g(1)*f(2)*g(3)*mm(ix  ,iy+1,iz  ) + f(1)*f(2)*g(3)*mm(ix+1,iy+1,iz  ) &
          &             + g(1)*g(2)*f(3)*mm(ix  ,iy  ,iz+1) + f(1)*g(2)*f(3)*mm(ix+1,iy  ,iz+1) &
          &             + g(1)*f(2)*f(3)*mm(ix  ,iy+1,iz+1) + f(1)*f(2)*f(3)*mm(ix+1,iy+1,iz+1)
      end if
    end do
  end subroutine lint3_lop2

  subroutine lint4_lop2(adj,mm,dd)
    logical,intent(in) :: adj  
    real, dimension (n(1),n(2),n(3),n(4))  :: mm 
    real, dimension (:)  :: dd
    integer :: id, im, ix, iy, iz, ia, i(dim)
    real    :: fract
    do id= 1,nd
      f = (xy(id,:)-o)/d
      i=f
      ix= 1+i(1)
      iy= 1+i(2)
      iz= 1+i(3)
      ia= 1+i(4)
      if ( 1>ix .or. ix>=n(1)) then
        cycle
      end if
      if ( 1>iy .or. iy>=n(2)) then
        cycle
      end if
      if ( 1>iz .or. iz>=n(3)) then
        cycle
      end if
      if ( 1>ia .or. ia>=n(4)) then
        cycle
      end if
      f=f-i
      g= 1.-f
      if ( adj) then
        mm(ix  ,iy  ,iz  ,ia  ) =            mm(ix  ,iy  ,iz  ,ia  ) + g(1)*g(2)*g(3)*g(4)*dd(id)
        mm(ix+1,iy  ,iz  ,ia  ) =            mm(ix+1,iy  ,iz  ,ia  ) + f(1)*g(2)*g(3)*g(4)*dd(id)
        mm(ix  ,iy+1,iz  ,ia  ) =            mm(ix  ,iy+1,iz  ,ia  ) + g(1)*f(2)*g(3)*g(4)*dd(id)
        mm(ix+1,iy+1,iz  ,ia  ) =            mm(ix+1,iy+1,iz  ,ia  ) + f(1)*f(2)*g(3)*g(4)*dd(id)
        mm(ix  ,iy  ,iz+1,ia  ) =            mm(ix  ,iy  ,iz+1,ia  ) + g(1)*g(2)*f(3)*g(4)*dd(id)
        mm(ix+1,iy  ,iz+1,ia  ) =            mm(ix+1,iy  ,iz+1,ia  ) + f(1)*g(2)*f(3)*g(4)*dd(id)
        mm(ix  ,iy+1,iz+1,ia  ) =            mm(ix  ,iy+1,iz+1,ia  ) + g(1)*f(2)*f(3)*g(4)*dd(id)
        mm(ix+1,iy+1,iz+1,ia  ) =            mm(ix+1,iy+1,iz+1,ia  ) + f(1)*f(2)*f(3)*g(4)*dd(id)
        mm(ix  ,iy  ,iz  ,ia+1) =            mm(ix  ,iy  ,iz  ,ia+1) + g(1)*g(2)*g(3)*f(4)*dd(id)
        mm(ix+1,iy  ,iz  ,ia+1) =            mm(ix+1,iy  ,iz  ,ia+1) + f(1)*g(2)*g(3)*f(4)*dd(id)
        mm(ix  ,iy+1,iz  ,ia+1) =            mm(ix  ,iy+1,iz  ,ia+1) + g(1)*f(2)*g(3)*f(4)*dd(id)
        mm(ix+1,iy+1,iz  ,ia+1) =            mm(ix+1,iy+1,iz  ,ia+1) + f(1)*f(2)*g(3)*f(4)*dd(id)
        mm(ix  ,iy  ,iz+1,ia+1) =            mm(ix  ,iy  ,iz+1,ia+1) + g(1)*g(2)*f(3)*f(4)*dd(id)
        mm(ix+1,iy  ,iz+1,ia+1) =            mm(ix+1,iy  ,iz+1,ia+1) + f(1)*g(2)*f(3)*f(4)*dd(id)
        mm(ix  ,iy+1,iz+1,ia+1) =            mm(ix  ,iy+1,iz+1,ia+1) + g(1)*f(2)*f(3)*f(4)*dd(id)
        mm(ix+1,iy+1,iz+1,ia+1) =            mm(ix+1,iy+1,iz+1,ia+1) + f(1)*f(2)*f(3)*f(4)*dd(id)

      else
        dd(id) = dd(id) + g(1)*g(2)*g(3)*g(4)*mm(ix  ,iy  ,iz  ,ia  ) &
          &             + f(1)*g(2)*g(3)*g(4)*mm(ix+1,iy  ,iz  ,ia  ) &
          &             + g(1)*f(2)*g(3)*g(4)*mm(ix  ,iy+1,iz  ,ia  ) &
          &             + f(1)*f(2)*g(3)*g(4)*mm(ix+1,iy+1,iz  ,ia  ) &
          &             + g(1)*g(2)*f(3)*g(4)*mm(ix  ,iy  ,iz+1,ia  ) &
          &             + f(1)*g(2)*f(3)*g(4)*mm(ix+1,iy  ,iz+1,ia  ) &
          &             + g(1)*f(2)*f(3)*g(4)*mm(ix  ,iy+1,iz+1,ia  ) &
          &             + f(1)*f(2)*f(3)*g(4)*mm(ix+1,iy+1,iz+1,ia  ) &
          &             + g(1)*g(2)*g(3)*f(4)*mm(ix  ,iy  ,iz  ,ia+1) &
          &             + f(1)*g(2)*g(3)*f(4)*mm(ix+1,iy  ,iz  ,ia+1) &
          &             + g(1)*f(2)*g(3)*f(4)*mm(ix  ,iy+1,iz  ,ia+1) &
          &             + f(1)*f(2)*g(3)*f(4)*mm(ix+1,iy+1,iz  ,ia+1) &
          &             + g(1)*g(2)*f(3)*f(4)*mm(ix  ,iy  ,iz+1,ia+1) &
          &             + f(1)*g(2)*f(3)*f(4)*mm(ix+1,iy  ,iz+1,ia+1) &
          &             + g(1)*f(2)*f(3)*f(4)*mm(ix  ,iy+1,iz+1,ia+1) &
          &             + f(1)*f(2)*f(3)*f(4)*mm(ix+1,iy+1,iz+1,ia+1)
      end if
    end do
  end subroutine lint4_lop2

  subroutine lint5_lop2(adj,mm,dd)
    logical,intent(in) :: adj  
    real, dimension (n(1),n(2),n(3),n(4),n(5))  :: mm 
    real, dimension (:)  :: dd
    integer :: id, im, ix, iy, iz, ia, ib, i(dim)
    real    :: fract
    do id= 1,nd
       f = (xy(id,:)-o)/d
       i=f
       ix= 1+i(1)
       iy= 1+i(2)
       iz= 1+i(3)
       ia= 1+i(4)
       ib= 1+i(5)
       if ( 1>ix .or. ix>=n(1)) then
          cycle
       end if
       if ( 1>iy .or. iy>=n(2)) then
          cycle
       end if
       if ( 1>iz .or. iz>=n(3)) then
          cycle
       end if
       if ( 1>ia .or. ia>=n(4)) then
          cycle
       end if
       if ( 1>ib .or. ib>=n(5)) then
          cycle
       end if
       f=f-i
       g= 1.-f
       if ( adj) then
          mm(ix  ,iy  ,iz  ,ia  ,ib  ) = mm(ix  ,iy  ,iz  ,ia  ,ib  ) + g(1)*g(2)*g(3)*g(4)*g(5)*dd(id)
          mm(ix+1,iy  ,iz  ,ia  ,ib  ) = mm(ix+1,iy  ,iz  ,ia  ,ib  ) + f(1)*g(2)*g(3)*g(4)*g(5)*dd(id)
          mm(ix  ,iy+1,iz  ,ia  ,ib  ) = mm(ix  ,iy+1,iz  ,ia  ,ib  ) + g(1)*f(2)*g(3)*g(4)*g(5)*dd(id)
          mm(ix+1,iy+1,iz  ,ia  ,ib  ) = mm(ix+1,iy+1,iz  ,ia  ,ib  ) + f(1)*f(2)*g(3)*g(4)*g(5)*dd(id)
          mm(ix  ,iy  ,iz+1,ia  ,ib  ) = mm(ix  ,iy  ,iz+1,ia  ,ib  ) + g(1)*g(2)*f(3)*g(4)*g(5)*dd(id)
          mm(ix+1,iy  ,iz+1,ia  ,ib  ) = mm(ix+1,iy  ,iz+1,ia  ,ib  ) + f(1)*g(2)*f(3)*g(4)*g(5)*dd(id)
          mm(ix  ,iy+1,iz+1,ia  ,ib  ) = mm(ix  ,iy+1,iz+1,ia  ,ib  ) + g(1)*f(2)*f(3)*g(4)*g(5)*dd(id)
          mm(ix+1,iy+1,iz+1,ia  ,ib  ) = mm(ix+1,iy+1,iz+1,ia  ,ib  ) + f(1)*f(2)*f(3)*g(4)*g(5)*dd(id)
          mm(ix  ,iy  ,iz  ,ia+1,ib  ) = mm(ix  ,iy  ,iz  ,ia+1,ib  ) + g(1)*g(2)*g(3)*f(4)*g(5)*dd(id)
          mm(ix+1,iy  ,iz  ,ia+1,ib  ) = mm(ix+1,iy  ,iz  ,ia+1,ib  ) + f(1)*g(2)*g(3)*f(4)*g(5)*dd(id)
          mm(ix  ,iy+1,iz  ,ia+1,ib  ) = mm(ix  ,iy+1,iz  ,ia+1,ib  ) + g(1)*f(2)*g(3)*f(4)*g(5)*dd(id)
          mm(ix+1,iy+1,iz  ,ia+1,ib  ) = mm(ix+1,iy+1,iz  ,ia+1,ib  ) + f(1)*f(2)*g(3)*f(4)*g(5)*dd(id)
          mm(ix  ,iy  ,iz+1,ia+1,ib  ) = mm(ix  ,iy  ,iz+1,ia+1,ib  ) + g(1)*g(2)*f(3)*f(4)*g(5)*dd(id)
          mm(ix+1,iy  ,iz+1,ia+1,ib  ) = mm(ix+1,iy  ,iz+1,ia+1,ib  ) + f(1)*g(2)*f(3)*f(4)*g(5)*dd(id)
          mm(ix  ,iy+1,iz+1,ia+1,ib  ) = mm(ix  ,iy+1,iz+1,ia+1,ib  ) + g(1)*f(2)*f(3)*f(4)*g(5)*dd(id)
          mm(ix+1,iy+1,iz+1,ia+1,ib  ) = mm(ix+1,iy+1,iz+1,ia+1,ib  ) + f(1)*f(2)*f(3)*f(4)*g(5)*dd(id)
          mm(ix  ,iy  ,iz  ,ia  ,ib+1) = mm(ix  ,iy  ,iz  ,ia  ,ib+1) + g(1)*g(2)*g(3)*g(4)*f(5)*dd(id)
          mm(ix+1,iy  ,iz  ,ia  ,ib+1) = mm(ix+1,iy  ,iz  ,ia  ,ib+1) + f(1)*g(2)*g(3)*g(4)*f(5)*dd(id)
          mm(ix  ,iy+1,iz  ,ia  ,ib+1) = mm(ix  ,iy+1,iz  ,ia  ,ib+1) + g(1)*f(2)*g(3)*g(4)*f(5)*dd(id)
          mm(ix+1,iy+1,iz  ,ia  ,ib+1) = mm(ix+1,iy+1,iz  ,ia  ,ib+1) + f(1)*f(2)*g(3)*g(4)*f(5)*dd(id)
          mm(ix  ,iy  ,iz+1,ia  ,ib+1) = mm(ix  ,iy  ,iz+1,ia  ,ib+1) + g(1)*g(2)*f(3)*g(4)*f(5)*dd(id)
          mm(ix+1,iy  ,iz+1,ia  ,ib+1) = mm(ix+1,iy  ,iz+1,ia  ,ib+1) + f(1)*g(2)*f(3)*g(4)*f(5)*dd(id)
          mm(ix  ,iy+1,iz+1,ia  ,ib+1) = mm(ix  ,iy+1,iz+1,ia  ,ib+1) + g(1)*f(2)*f(3)*g(4)*f(5)*dd(id)
          mm(ix+1,iy+1,iz+1,ia  ,ib+1) = mm(ix+1,iy+1,iz+1,ia  ,ib+1) + f(1)*f(2)*f(3)*g(4)*f(5)*dd(id)
          mm(ix  ,iy  ,iz  ,ia+1,ib+1) = mm(ix  ,iy  ,iz  ,ia+1,ib+1) + g(1)*g(2)*g(3)*f(4)*f(5)*dd(id)
          mm(ix+1,iy  ,iz  ,ia+1,ib+1) = mm(ix+1,iy  ,iz  ,ia+1,ib+1) + f(1)*g(2)*g(3)*f(4)*f(5)*dd(id)
          mm(ix  ,iy+1,iz  ,ia+1,ib+1) = mm(ix  ,iy+1,iz  ,ia+1,ib+1) + g(1)*f(2)*g(3)*f(4)*f(5)*dd(id)
          mm(ix+1,iy+1,iz  ,ia+1,ib+1) = mm(ix+1,iy+1,iz  ,ia+1,ib+1) + f(1)*f(2)*g(3)*f(4)*f(5)*dd(id)
          mm(ix  ,iy  ,iz+1,ia+1,ib+1) = mm(ix  ,iy  ,iz+1,ia+1,ib+1) + g(1)*g(2)*f(3)*f(4)*f(5)*dd(id)
          mm(ix+1,iy  ,iz+1,ia+1,ib+1) = mm(ix+1,iy  ,iz+1,ia+1,ib+1) + f(1)*g(2)*f(3)*f(4)*f(5)*dd(id)
          mm(ix  ,iy+1,iz+1,ia+1,ib+1) = mm(ix  ,iy+1,iz+1,ia+1,ib+1) + g(1)*f(2)*f(3)*f(4)*f(5)*dd(id)
          mm(ix+1,iy+1,iz+1,ia+1,ib+1) = mm(ix+1,iy+1,iz+1,ia+1,ib+1) + f(1)*f(2)*f(3)*f(4)*f(5)*dd(id)
          
       else
          dd(id) = dd(id) + g(1)*g(2)*g(3)*g(4)*g(5)*mm(ix  ,iy  ,iz  ,ia  ,ib  ) &
            &             + f(1)*g(2)*g(3)*g(4)*g(5)*mm(ix+1,iy  ,iz  ,ia  ,ib  ) &
            &             + g(1)*f(2)*g(3)*g(4)*g(5)*mm(ix  ,iy+1,iz  ,ia  ,ib  ) &
            &             + f(1)*f(2)*g(3)*g(4)*g(5)*mm(ix+1,iy+1,iz  ,ia  ,ib  ) &
            &             + g(1)*g(2)*f(3)*g(4)*g(5)*mm(ix  ,iy  ,iz+1,ia  ,ib  ) &
            &             + f(1)*g(2)*f(3)*g(4)*g(5)*mm(ix+1,iy  ,iz+1,ia  ,ib  ) &
            &             + g(1)*f(2)*f(3)*g(4)*g(5)*mm(ix  ,iy+1,iz+1,ia  ,ib  ) &
            &             + f(1)*f(2)*f(3)*g(4)*g(5)*mm(ix+1,iy+1,iz+1,ia  ,ib  ) &
            &             + g(1)*g(2)*g(3)*f(4)*g(5)*mm(ix  ,iy  ,iz  ,ia+1,ib  ) &
            &             + f(1)*g(2)*g(3)*f(4)*g(5)*mm(ix+1,iy  ,iz  ,ia+1,ib  ) &
            &             + g(1)*f(2)*g(3)*f(4)*g(5)*mm(ix  ,iy+1,iz  ,ia+1,ib  ) &
            &             + f(1)*f(2)*g(3)*f(4)*g(5)*mm(ix+1,iy+1,iz  ,ia+1,ib  ) &
            &             + g(1)*g(2)*f(3)*f(4)*g(5)*mm(ix  ,iy  ,iz+1,ia+1,ib  ) &
            &             + f(1)*g(2)*f(3)*f(4)*g(5)*mm(ix+1,iy  ,iz+1,ia+1,ib  ) &
            &             + g(1)*f(2)*f(3)*f(4)*g(5)*mm(ix  ,iy+1,iz+1,ia+1,ib  ) &
            &             + f(1)*f(2)*f(3)*f(4)*g(5)*mm(ix+1,iy+1,iz+1,ia+1,ib  ) &
            &             + g(1)*g(2)*g(3)*g(4)*f(5)*mm(ix  ,iy  ,iz  ,ia  ,ib+1) &
            &             + f(1)*g(2)*g(3)*g(4)*f(5)*mm(ix+1,iy  ,iz  ,ia  ,ib+1) &
            &             + g(1)*f(2)*g(3)*g(4)*f(5)*mm(ix  ,iy+1,iz  ,ia  ,ib+1) &
            &             + f(1)*f(2)*g(3)*g(4)*f(5)*mm(ix+1,iy+1,iz  ,ia  ,ib+1) &
            &             + g(1)*g(2)*f(3)*g(4)*f(5)*mm(ix  ,iy  ,iz+1,ia  ,ib+1) &
            &             + f(1)*g(2)*f(3)*g(4)*f(5)*mm(ix+1,iy  ,iz+1,ia  ,ib+1) &
            &             + g(1)*f(2)*f(3)*g(4)*f(5)*mm(ix  ,iy+1,iz+1,ia  ,ib+1) &
            &             + f(1)*f(2)*f(3)*g(4)*f(5)*mm(ix+1,iy+1,iz+1,ia  ,ib+1) &
            &             + g(1)*g(2)*g(3)*f(4)*f(5)*mm(ix  ,iy  ,iz  ,ia+1,ib+1) &
            &             + f(1)*g(2)*g(3)*f(4)*f(5)*mm(ix+1,iy  ,iz  ,ia+1,ib+1) &
            &             + g(1)*f(2)*g(3)*f(4)*f(5)*mm(ix  ,iy+1,iz  ,ia+1,ib+1) &
            &             + f(1)*f(2)*g(3)*f(4)*f(5)*mm(ix+1,iy+1,iz  ,ia+1,ib+1) &
            &             + g(1)*g(2)*f(3)*f(4)*f(5)*mm(ix  ,iy  ,iz+1,ia+1,ib+1) &
            &             + f(1)*g(2)*f(3)*f(4)*f(5)*mm(ix+1,iy  ,iz+1,ia+1,ib+1) &
            &             + g(1)*f(2)*f(3)*f(4)*f(5)*mm(ix  ,iy+1,iz+1,ia+1,ib+1) &
            &             + f(1)*f(2)*f(3)*f(4)*f(5)*mm(ix+1,iy+1,iz+1,ia+1,ib+1)
         end if
      end do
    end subroutine lint5_lop2
    
    subroutine lint_close()
      deallocate(n,o,d,f,g)
    end subroutine lint_close
  end module lint
