module mpef_find
! Find multi-scale prediction-error filter (helix magic)
  use rescale_bill
  use mhconest_new
  use hconest
  use cgstep_mod
  use solver_smp_mod
  use dottest
  use error_check
  use fill_s
  implicit none
contains

  subroutine mpef_find(data,   mask,n,o,   d,a,   center,   gap,   npatch,   niter,eps,prec,sc,jump,aa,epsl,na)
    real,pointer    ::  data(:),mask(:)
    integer         ::               n(:),                                                   sc(:),jump(:)
    real            ::                 o(:),d(:),                                   eps,           epsl
    integer         ::                        a(:),center(:),gap(:),npatch(:),niter,    prec,           na,bsize
    type(nfilter)   ::                                                                          aa
    real,   pointer :: pefcoeffs(:),databank(:),obank(:,:),dbank(:,:)
    integer,pointer :: nbank(:,:),lagbank(:),mark(:),patchbank(:)
    logical,pointer :: boundbank(:),maskbank(:)
    real, dimension(2):: dot1,dot2

! __________________________________________________________
! ___________Find scale coordinates, size of arrays_________
! __________________________________________________________
    allocate(nbank(size(n),size(sc)),mark(size(sc)+1))
    allocate(obank(size(nbank,1),size(nbank,2)))
    allocate(dbank(size(nbank,1),size(nbank,2)))
    call find_nodbank(n,o,d,sc,nbank,obank,dbank)
    mark   = find_mark (nbank)
    bsize  = mark(size(nbank,2)+1)-1
    allocate(lagbank(size(sc)*size(aa%hlx(1)%lag)),lagmark(size(sc)+1))
!    call imms(lagmark,"lmark")
!    write(0,*)"find"
    call lagbank_find(nbank,a,center,gap,lagbank,lagmark)
! __________________________________________________________
! ___________Allocate space for info on scales______________
! __________________________________________________________
!    write(0,*)"bsize",bsize
    allocate(maskbank(bsize))
    allocate(databank(bsize))
    allocate(boundbank(bsize))
! __________________________________________________________
! ___________Precompute PEF info for different scales_______
! __________________________________________________________

    call maskbank_find(mask,n,o,d,nbank,obank,dbank,maskbank)
!call rmms(data,"data ")

    call databank_find(data,mask,n,o,d,nbank,obank,dbank,databank)
!call rmms(databank,"dbank")
    call boundbank_find(maskbank,nbank,a,center,gap,boundbank)

allocate(pefcoeffs(lagmark(2)-1))
! __________________________________________________________
! ___________Pass precomputed stuff to operator init________
! __________________________________________________________

!call rmms(databank,"dbank")
    call mhconest_new_init(databank,boundbank,lagbank,mark,lagmark,aa)

! __________________________________________________________
! ___________Call solver for either preconditioned or_______
! ___________just regularized case._________________________
! __________________________________________________________


!    call solver_smp(m=aa%flt,             &
!                    d=-databank,          &
!                    Fop=mhconest_new_lop, &
!                    stepper=cgstep,       &
!                    niter=niter,          &
!                    verb=verb)

    call dot_test(mhconest_new_lop,size(aa%flt),size(databank),dot1,dot2)
    call mhconest_new_close()
    call cgstep_close()
    
! __________________________________________________________
! ___________Re-initialize PEF for original data____________
! __________________________________________________________
    call set_pef(aa,patchbank,lagbank,boundbank,pefcoeffs,mark(2)-1,lagmark(2)-1)
! __________________________________________________________
! ___________Deallocate bookkeeping devices for scales______
! __________________________________________________________

    deallocate(maskbank,databank,boundbank,lagbank,dbank,obank)
  end subroutine mpef_find

end module mpef_find
