module nlnmpef_find
! Find multi-scale prediction-error filter (helix magic)
  use rescale_bill
  use nmhconest_new
  use nhconest
  use cgstep_mod
  use solver_prc_mod
  use solver_reg_mod
  use ilaplacf
  use laplacf
  use dottest
  use error_check
  use fill_ns
  implicit none
contains
  
  subroutine nlnmpef_find(pef,data,   mask,n,o,   d,a,   center,   gap,   npatch,   niter,eps,prec,sc,bb,epsl,na,nliter,feps,fiter)
    real,pointer    ::        data(:),mask(:)
    integer         ::                     n(:),                                                   sc(:),i
    real            ::                       o(:),d(:),                                   eps,           epsl,          feps
    integer         :: bsize,                       a(:),center(:),gap(:),npatch(:),niter,    prec,           na,nliter,     fiter
    type(nfilter)   ::    pef,                                                                        bb
    real,   pointer :: pefcoeffs(:),databank(:),obank(:,:),dbank(:,:)
    integer,allocatable:: temppatchbank(:)
    integer,pointer :: nbank(:,:),lagbank(:),mark(:),patchbank(:)
    logical,pointer :: boundbank(:),maskbank(:)
    real, dimension(2):: dot1,dot2

!write(0,*)"nlnmpef_begin"
! __________________________________________________________
! ___________Find scale coordinates, size of arrays_________
! __________________________________________________________
    allocate(nbank(size(n),size(sc)),obank(size(n),size(sc)),dbank(size(n),size(sc)),mark(size(sc)+1))
    call find_nodbank(n,o,d,sc,nbank,obank,dbank)
    mark   = find_mark (nbank)
    bsize  = mark(size(nbank,2)+1)-1
    allocate(lagbank(size(sc)*size(bb%hlx(1)%lag)),lagmark(size(sc)+1))
    call lagbank_find(nbank,a,center,gap,lagbank,lagmark)
! __________________________________________________________
! ___________Allocate space for info on scales______________
! __________________________________________________________
!write(0,*)"nlnmpef 2"

    allocate(maskbank(bsize))
    allocate(databank(bsize))
    allocate(patchbank(bsize))
    allocate(boundbank(bsize))
! __________________________________________________________
! ___________Precompute PEF info for different scales_______
! __________________________________________________________
!write(0,*)"nlnmpef 3"

    maskbank=.true.
!call lmms(maskbank,"mbank")
    call boundbank_find(maskbank,nbank,a,center,gap,boundbank)
    call micropatchbank_find(npatch,nbank,patchbank)
!call imms(patchbank,"pbank")
!write(0,*)"nlnmpef 4"
allocate(pefcoeffs((lagmark(2)-1)*maxval(patchbank)))
!call imms(patchbank,"pbank")
! __________________________________________________________
! ___________Pass precomputed stuff to operator init________
! __________________________________________________________
    do i=1,nliter
write(0,*)"NONLINEAR ITERATION #",i,nliter
       allocate(temppatchbank(size(patchbank)))
       temppatchbank=patchbank

       call databank_pef_find(pef,data,mask,n,o,d,nbank,obank,dbank,feps,fiter,databank)

       if (allocated(patchbank)) deallocate(patchbank)
       call imms(temppatchbank,"tbank")
       allocate(patchbank(size(temppatchbank)))
       patchbank=temppatchbank
       deallocate(temppatchbank)

       call nmhconest_new_init(databank,patchbank,boundbank,lagbank,mark,lagmark,bb)

       if (allocated(pef%mis)) deallocate(pef%mis)
       if (allocated(bb%mis)) deallocate(bb%mis)

       call set_pef(bb,patchbank,lagbank,boundbank,pefcoeffs,mark(2)-1,lagmark(2)-1)
       call set_pef(pef,patchbank,lagbank,boundbank,pefcoeffs,mark(2)-1,lagmark(2)-1)

! __________________________________________________________
! ___________Call solver for either preconditioned or_______
! ___________just regularized case._________________________
! __________________________________________________________

       if (prec==1) then
          call ilaplacf_init(npatch,lagmark(2)-1,epsl,na)
          !      call dot_test(nmhconest_new_lop,size(pefcoeffs),size(databank),dot1,dot2)
          !      call dot_test(ilaplacf_lop,size(pefcoeffs),size(pefcoeffs),dot1,dot2)
          call solver_prc(m=pefcoeffs,                               &
               d=-databank,                               &
               Fop=nmhconest_new_lop,                      &
               Sop=ilaplacf_lop,                           &
               stepper=cgstep,                             &
               nSop=(product(npatch)*size(bb%hlx(1)%flt)), &
               niter=niter,                                &
               eps=eps,                                    &
               verb=.true.)
write(0,*)"before ilap_close"
          call ilaplacf_close()
       else
          call laplacf_init(npatch,lagmark(2)-1)
!                call dot_test(nmhconest_new_lop,size(pefcoeffs),size(databank),dot1,dot2)
!                call dot_test(laplacf_lop,size(pefcoeffs),size(pefcoeffs),dot1,dot2)
          call solver_reg(m=pefcoeffs,                                &
                          d=-databank,                                &
                          Fop=nmhconest_new_lop,                      &
                          Aop=laplacf_lop,                            &
                          stepper=cgstep,                             &
                          nAop=(product(npatch)*size(bb%hlx(1)%flt)), &
                          niter=niter,                                &
                          eps=eps,                                    &
                          verb=.true.)
          call laplacf_close()
       end if
write(0,*)"before nmh_close"
       call nmhconest_new_close()
       call cgstep_close()
write(0,*)"after solver",i
call rmms(pefcoeffs,"pefca")
write(0,*)"PEF: bb"
call pefcheck(bb)
write(0,*)"PEF: pef"
call pefcheck(pef)       

call imms(patchbank,"pbank")

! __________________________________________________________
! ___________Re-initialize PEF for original data____________
! __________________________________________________________
       call set_pef(bb,patchbank,lagbank,boundbank,pefcoeffs,mark(2)-1,lagmark(2)-1)
       call set_pef(pef,patchbank,lagbank,boundbank,pefcoeffs,mark(2)-1,lagmark(2)-1)
call pefcheck(bb)
       databank=0.
       pefcoeffs=0.
       pef=bb
    end do
!       call set_pef(aa,patchbank,lagbank,boundbank,pefcoeffs,mark(2)-1,lagmark(2)-1)
write(0,*)"after PEF: bb"
call pefcheck(bb)
! __________________________________________________________
! ___________Deallocate bookkeeping devices for scales______
! __________________________________________________________

    deallocate(maskbank,databank,boundbank,lagbank,patchbank,dbank,obank,pefcoeffs)
  end subroutine nlnmpef_find

  subroutine nlnpef_find(pef,data,   mask,n,o,   d,a,   center,   gap,   npatch,   niter,eps,prec,bb,epsl,na,nliter,feps,fiter)
    real,pointer    ::        data(:),mask(:)
    integer         ::                     n(:),                                                   sc(:),i,j,k
    real            ::                       o(:),d(:),                                   eps,           epsl,          feps
    integer         :: bsize,                       a(:),center(:),gap(:),npatch(:),niter,    prec,           na,nliter,     fiter
    type(nfilter)   ::    pef,                                                                        bb
    real,   pointer :: pefcoeffs(:),filldata(:)
    real, dimension(2):: dot1,dot2

! __________________________________________________________
! ___________Pass precomputed stuff to operator init________
! __________________________________________________________
    do i=1,nliter
write(0,*)"NONLINEAR ITERATION #",i,nliter
       allocate(filldata(size(data)),pefcoeffs(size(bb%hlx(1)%lag)*maxval(bb%pch)))

       filldata=scale_data_pef(pef,data,mask,n,o,d,n,o,d,eps,fiter)
write(0,*)"PEF bb"
call pefcheck(bb)
       call nhconest_init( filldata, bb,size(bb%hlx(1)%flt) )

! __________________________________________________________
! ___________Call solver for either preconditioned or_______
! ___________just regularized case._________________________
! __________________________________________________________

       if (prec==1) then
          call ilaplacf_init(npatch,size(bb%hlx(1)%lag),epsl,na)
          !      call dot_test(nhconest_lop,size(pefcoeffs),size(filldata),dot1,dot2)
          !      call dot_test(ilaplacf_lop,size(pefcoeffs),size(pefcoeffs),dot1,dot2)
          call solver_prc(m=pefcoeffs,                     &
               d=-filldata ,                               &
               Fop=nhconest_lop,                           &
               Sop=ilaplacf_lop,                           &
               stepper=cgstep,                             &
               nSop=(maxval(bb%pch)*size(bb%hlx(1)%flt)),  &
               niter=niter,                                &
               eps=eps,                                    &
               verb=.true.)
write(0,*)"before ilap_close"
          call ilaplacf_close()
       else
          call laplacf_init(npatch,lagmark(2)-1)
!                call dot_test(nhconest_lop,size(pefcoeffs),size(filldata),dot1,dot2)
!                call dot_test(laplacf_lop,size(pefcoeffs),size(pefcoeffs),dot1,dot2)
          call solver_reg(m=pefcoeffs,                                &
                          d=-filldata,                                &
                          Fop=nhconest_lop,                           &
                          Aop=laplacf_lop,                            &
                          stepper=cgstep,                             &
                          nAop=(maxval(bb%pch)*size(bb%hlx(1)%flt)),  &
                          niter=niter,                                &
                          eps=eps,                                    &
                          verb=.true.)
          call laplacf_close()
       end if
write(0,*)"before nmh_close"
       call nhconest_close()
       call cgstep_close()
write(0,*)"after solver",i
call rmms(pefcoeffs,"pefca")
       k=1
       do j=1,maxval(bb%pch)
          bb%hlx(j)%flt=pefcoeffs(k:k+size(bb%hlx(j)%lag))
          k=k+size(bb%hlx(j)%lag)
       end do
write(0,*)"PEF: bb"
call pefcheck(bb)
write(0,*)"PEF: pef"
call pefcheck(pef)       

call pefcheck(bb)
       pefcoeffs=0.
       pef=bb
    end do
write(0,*)"after PEF: bb"
call pefcheck(bb)
! __________________________________________________________
! ___________Deallocate bookkeeping devices for scales______
! __________________________________________________________

    deallocate(filldata,pefcoeffs)
  end subroutine nlnpef_find

end module nlnmpef_find
