module nmhconest_shift

use nhelix
use nhconest
use sep
use adj_mod
use regridn        

implicit none

type(nfilter), private, save         :: aa
integer, private, save               :: dim, ns, stat
real, dimension(:), pointer, save    :: dfg, ofg, databank
integer, dimension(:), pointer, save :: oldpatch, patchbank, nsh, nag, nsg, mark, lagbank, lagmark
logical, dimension(:), pointer, save :: misbank
integer, dimension(:,:), pointer, save :: nbank, tsh

contains

  subroutine nmhconest_shift_init( databankin, aa0, markin, nshin, nbankin, df, of, na, misbankin )
    type(nfilter)                    :: aa0
    real, dimension(:)               :: df, of
    real, dimension(:), pointer      :: rd, ro, databankin
    integer, dimension(:,:)          :: nbankin
    integer, dimension(:)            :: markin, nshin
    integer, dimension(:), pointer   :: na, newcart, patch, oldcart
    integer                          :: datsize, is, id, ip, banksize, oldline, stat, neq, nlag, k, ih
    logical, dimension(:)            :: misbankin

                                                                               write(0,*)"size1 init"
    dim=size(nbankin,2)
                                                                               write(0,*)"size2 init dim", dim
    ns = size(nbankin,1)
                                                                               write(0,*)"allocate ns", ns
    allocate( misbank(size(misbankin)),databank(size(databankin)),oldpatch(size(aa0%pch)), mark(size(markin)), lagmark(ns+1))
    allocate( dfg(dim), ofg(dim), nag(dim), nsg(dim),newcart(dim),oldcart(dim), nbank(ns,dim), nsh(ns),tsh(size(markin)-1,dim))
    aa = aa0; misbank = misbankin; databank=databankin; mark=markin
    dfg = df; ofg = of; nag = na; nbank=nbankin; nsh=nshin

    oldpatch = aa%pch
    lagmark(1)=1
                                                                               write(0,*)"setup lagmark"
    do is=2,ns+1
       lagmark(is) = lagmark(is-1) + size(aa%hlx(1)%lag)
    end do
                                                                               write(0,*)"after lagmark",lagmark
                                                                               write(0,*)"setup mark", mark

    banksize = mark(size(mark))-1
                                                                               write(0,*)"banksize",banksize
    allocate(patchbank(banksize))
                                                                               write(0,*)"size5 init",ns,size(aa%hlx(1)%lag)
    allocate(lagbank(size(aa%hlx(1)%lag)*ns))
    lagbank = 0

    if (allocated(aa%mis)) then 
       deallocate(aa%mis)
    endif
    if (allocated(aa%pch)) then 
       deallocate(aa%pch)
    endif


write(0,*)"init 6"
    k=1
! Loop over scales, set up everything
    do is=1,ns
       do ih=0,nsh(is)-1
          allocate(patch(product(nbank(is,:))))
          ! set up patchbank
          tsh=0
          do id=2,dim
             write(0,*)"break4,ih,dim,id k",ih,dim,id,k
             tsh(k,id)=mod(int(ih/(2**(dim-id))),2)
          end do
          
             write(0,*)"break5,ih,id,k,tsh",ih,id,k,tsh(k,:)
          do ip=1,product(nbank(is,:))
             call line2cart(nbank(is,:),ip,newcart)
             do id=1,dim
                if (newcart(id)==nbank(is,id)) then
                   oldcart(id) = nint(real(newcart(id))*real(nbank(1,id))/real(nbank(is,id)))
                   else
                      oldcart(id) = nint(real(newcart(id))*real(nbank(1,id))/real(nbank(is,id)))+tsh(k,id)
                   end if
             end do
             call cart2line(nbank(1,:),oldcart,oldline)
             patch(ip) = oldpatch(oldline)
          end do
             write(0,*)"break6,k mark(k), mark(k+1),sz patchbank",k,mark(k),mark(k+1),size(patchbank)
       patchbank(mark(k):mark(k+1)-1) = patch
             write(0,*)"break7",maxval(patch),size(patch),maxval(patchbank(mark(k):mark(k+1)-1)),size(patchbank(mark(k):mark(k+1)-1))
       deallocate(patch)
             write(0,*)"break8"
       k=k+1
    end do
             write(0,*)"break9",mark(k-1),mark(k)-1, size(patchbank),product(nbank(is,:))
    allocate(aa%pch(product(nbank(is,:))))
             write(0,*)"break10",maxval(patchbank(mark(k-1):mark(k)-1)),size(patchbank(mark(k-1):mark(k)-1))
    ! set up lagbank
    aa%pch=patchbank(mark(k-1):mark(k)-1)
             write(0,*)"break11",maxval(aa%pch), size( aa%hlx), nbank(1,:),nbank(is,:)
    call nregridn(nbank(1,:),nbank(is,:),aa) 
             write(0,*)"break12"
    lagbank(lagmark(is):lagmark(is+1)-1)=aa%hlx(1)%lag
             write(0,*)"break13"
    call nregridn(nbank(is,:),nbank(1,:),aa)
 end do
    neq=count(.not. misbank)
     write(0,*)"Number of fitting equations:",neq
  end subroutine nmhconest_shift_init
! ---------------------------------------------------------------------------
! ---------------------------------------------------------------------------    
       
  function nmhconest_shift_lop ( adj, add, mm, dd) result(stat)
    integer            :: stat 
    logical,intent(in) :: adj,add 
    real,dimension(:)  :: mm,dd 
    call adjnull (adj,add,mm,dd )
    call nmhconest_shift_lop2(adj,mm,dd )
    stat=0
  end function 

  subroutine nmhconest_shift_lop2( adj, mm, dd)
    logical                        :: adj 
    real, dimension(:)             :: mm, dd
    real, dimension(:), pointer    :: datatmp, datar
    integer, dimension(dim)        :: rn
    integer                        :: is, datsize, ia, k, ih

    if (allocated(aa%pch)) then
       deallocate(aa%pch)
    endif
    if (allocated(aa%mis)) then
       deallocate(aa%mis)
    endif
!                                 write(0,*)"lop 1",maxval(patchbank),size(patchbank),mark
    k=1
    do is=1,ns                                      ! looping over scales
!                                 write(0,*)"lop 2"
       do ih=1,nsh(is)
!                                 write(0,*)"lop 3"
          datsize = product(nbank(is,:))
!                                 write(0,*)"lop 3.1"
          allocate(aa%mis(datsize),aa%pch(datsize),datatmp(datsize),datar(datsize))
!                                 write(0,*)"lop 3.2"
          datatmp = 0.0
          aa%mis = misbank(mark(k):mark(k+1)-1)
!                                 write(0,*)"lop 3.3"
          aa%pch = patchbank(mark(k):mark(k+1)-1)
!                                 write(0,*)"lop 3.4",maxval(aa%pch),size(aa%hlx(1)%lag),size(lagbank(lagmark(is):lagmark(is+1)-1))
          do ia=1,maxval(aa%pch)
             aa%hlx(ia)%lag=lagbank(lagmark(is):lagmark(is+1)-1)
          end do
!                                 write(0,*)"lop 3.5"
          datar = databank(mark(k):mark(k+1)-1)
!                                 write(0,*)"lop 3.6"
          call nhconest_init( datar, aa,size(aa%hlx(1)%flt) )
!                                 write(0,*)"lop 3.7"
          datatmp = dd(mark(k):mark(k+1)-1)
!                                 write(0,*)"lop 3.8"
          stat = nhconest_lop( adj, .true., mm, datatmp )  ! Just normal nhconest--note add=.true.
!                                 write(0,*)"lop 3.9"
          dd(mark(k):mark(k+1)-1) = datatmp
!                                 write(0,*)"lop 3.0"
          deallocate( datatmp, datar)
          deallocate(aa%pch,aa%mis)
          k=k+1
       end do
    end do
  end subroutine nmhconest_shift_lop2
! ---------------------------------------------------------------------------

! ---------------------------------------------------------------------------
  subroutine nmhconest_shift_close( )
		deallocate(databank,nbank,dfg,ofg,nag,oldpatch,patchbank,misbank,lagbank)
  end subroutine nmhconest_shift_close
! ---------------------------------------------------------------------------

end module nmhconest_shift







