module nmpef_find_old
! Find multi-scale prediction-error filter (helix magic)
  use rescale_bill
  use nmhconest_new
  use cgstep_mod
  use solver_prc_mod
  use solver_reg_mod
  use ilaplacf
  use laplacf
  use dottest
  use error_check

  implicit none
contains
  
  subroutine nmpef_find_old(data,   mask,n,o,   d,a,   center,   gap,   npatch,   niter,eps,prec,sc,aa,epsl,na)
    real,pointer    ::  data(:),mask(:)
    integer         ::               n(:),                                                   sc(:)
    real            ::                 o(:),d(:),                                   eps,           epsl
    integer         ::                        a(:),center(:),gap(:),npatch(:),niter,    prec,           na,bsize
    type(nfilter)   ::                                                                       aa
    real,   pointer :: pefcoeffs(:),databank(:),obank(:,:),dbank(:,:)
    integer,pointer :: nbank(:,:),lagbank(:),mark(:),patchbank(:)
    logical,pointer :: boundbank(:),maskbank(:)
    real, dimension(2):: dot1,dot2
! __________________________________________________________
! ___________Find scale coordinates, size of arrays_________
! __________________________________________________________
    allocate(nbank(size(n),size(sc)),mark(size(sc)+1))
    allocate(obank(size(n),size(sc)),dbank(size(n),size(sc)))

    call find_nodbank(n,o,d,sc,nbank,obank,dbank)
    mark   = find_mark(nbank)
    bsize  = mark(size(nbank,2)+1)-1

    allocate(lagbank(size(sc)*size(aa%hlx(1)%lag)),lagmark(size(sc)))

    call lagbank_find(nbank,a,center,gap,lagbank,lagmark)

! __________________________________________________________
! ___________Allocate space for info on scales______________
! __________________________________________________________

    allocate(maskbank(mark(size(mark))-1),databank(mark(size(mark))-1))
    allocate(boundbank(mark(size(mark))-1),patchbank(mark(size(mark))-1))
! __________________________________________________________
! ___________Precompute PEF info for different scales_______
! __________________________________________________________
write(0,*)"bank 1"
    call maskbank_find(mask,n,o,d,nbank,obank,dbank,maskbank)
call lmms(maskbank,"mbank")
write(0,*)"bank 2"
    call databank_find(data,mask,n,o,d,nbank,obank,dbank,databank)
write(0,*)"bank 3"
call rmms(databank,"dbank")
    call boundbank_find(maskbank,nbank,a,center,gap,boundbank)
write(0,*)"bank 4"
call lmms(boundbank,"bbank")
    call micropatchbank_find(npatch,nbank,patchbank)
write(0,*)"bank 5"
call imms(patchbank,"pbank")

    allocate(pefcoeffs((lagmark(2)-1)*maxval(patchbank)))
! __________________________________________________________
! ___________Pass precomputed stuff to operator init________
! __________________________________________________________

    call nmhconest_new_init(databank,patchbank,boundbank,lagbank,mark,lagmark,aa)

! __________________________________________________________
! ___________Call solver for either preconditioned or_______
! ___________just regularized case._________________________
! __________________________________________________________

    if (prec==1) then
       call ilaplacf_init(npatch,lagmark(2)-1,epsl,na)
!      call dot_test(nmhconest_new_lop,size(pefcoeffs),size(databank),dot1,dot2)
!      call dot_test(ilaplacf_lop,size(pefcoeffs),size(pefcoeffs),dot1,dot2)
       call solver_prc(m=pefcoeffs,                               &
                       d=-databank,                               &
                      Fop=nmhconest_new_lop,                      &
                      Sop=ilaplacf_lop,                           &
                      stepper=cgstep,                             &
                      nSop=(product(npatch)*size(aa%hlx(1)%flt)), &
                      niter=niter,                                &
                      eps=eps,                                    &
                      verb=.true.)
       call ilaplacf_close()
    else
       call laplacf_init(npatch,lagmark(2)-1)
!      call dot_test(nmhconest_new_lop,size(pefcoeffs),size(databank),dot1,dot2)
!      call dot_test(laplacf_lop,size(pefcoeffs),size(pefcoeffs),dot1,dot2)
       call solver_reg(m=pefcoeffs,                               &
                       d=-databank,                               &
                      Fop=nmhconest_new_lop,                      &
                      Aop=laplacf_lop,                            &
                      stepper=cgstep,                             &
                      nAop=(product(npatch)*size(aa%hlx(1)%flt)), &
                      niter=niter,                                &
                      eps=eps,                                    &
                      verb=.true.)
       call laplacf_close()
    end if
    call nmhconest_new_close()
    call cgstep_close()
    
! __________________________________________________________
! ___________Re-initialize PEF for original data____________
! __________________________________________________________
    call set_pef(aa,patchbank,lagbank,boundbank,pefcoeffs,mark(2)-1,lagmark(2)-1)
! __________________________________________________________
! ___________Deallocate bookkeeping devices for scales______
! __________________________________________________________

    deallocate(maskbank,databank,boundbank,lagbank,patchbank,nbank,dbank,obank,pefcoeffs)
  end subroutine nmpef_find

end module nmpef_find
