module nmpef_find_sean
! Find multi-scale prediction-error filter (helix magic)
  use rescale_bill
  use nmhconest_sean
  use nhconest
  use cgstep_mod
  use solver_prc_mod
  use solver_reg_mod
  use ilaplacf
  use laplacf
  use dottest
  use error_check
  use fill_ns
  implicit none
contains

  subroutine nmpef_find_sean(data,   mask,n,o,   d,a,   center,   gap,   npatch,   niter,eps,prec,sc,   jump,aa,epsl,na)
    real,pointer    ::       data(:),mask(:)
    integer         ::                    n(:),                                                   sc(:),jump(:)
    real            ::                      o(:),d(:),                                   eps,                   epsl
    integer         ::                             a(:),center(:),gap(:),npatch(:),niter,    prec,                   na,bsize,i,j,nlag
    type(nfilter)   ::                                                                                       aa
    real,   pointer :: pefcoeffs(:)
    integer,pointer :: lagbank(:),mark(:),jbank(:,:)
    logical,pointer :: boundbank(:),lmask(:)
    real, dimension(2):: dot1,dot2

! __________________________________________________________
! ___________Find scale coordinates, size of arrays_________
! __________________________________________________________

write(0,*)"nmpef_sean 1"

    allocate(lagbank(size(sc)*size(aa%hlx(1)%lag)),lagmark(size(sc)+1),jbank(size(sc),size(jump)),lmask(size(mask)))
write(0,*)"nmpef_sean 2"
    lagmark(1)=1
    do i=1,size(sc)
       do j=1,size(jump)
          jbank(i,j)=sc(i)*jump(j)
       end do
       lagmark(i+1)=lagmark(i)+size(aa%hlx(1)%lag)
    end do
    nlag=size(aa%hlx(1)%lag)
write(0,*)"nmpef_sean 3",jbank
    lmask=.false.
    where (mask/=0)
       lmask=.true.
    end where
write(0,*)"nmpef_sean 4"
    call imms(lagmark,"lmark")
!    write(0,*)"find"
    call lagbank_jump_find(jbank,n,a,center,gap,size(aa%hlx(1)%lag),lagbank)

    call createpatch(npatch,n,aa%pch)
! __________________________________________________________
! ___________Allocate space for info on scales______________
! __________________________________________________________
!    write(0,*)"bsize",bsize
write(0,*)"nmpef_sean 5",lagbank
    allocate(boundbank(size(mask)*size(sc)))
! __________________________________________________________
! ___________Precompute PEF info for different scales_______
! __________________________________________________________

write(0,*)"nmpef_sean 6"
    call boundbank_sean_find(n,lmask,lagbank,nlag,a,center,gap,boundbank)

write(0,*)"nmpef_sean 7"
allocate(pefcoeffs((lagmark(2)-1)*maxval(aa%pch)))
! __________________________________________________________
! ___________Pass precomputed stuff to operator init________
! __________________________________________________________
call imms(aa%pch,"aapch")
write(0,*)"nmpef_sean 8"
call imms(lagbank,"lbank")
!call rmms(databank,"dbank")
    call nmhconest_sean_init(data,boundbank,lagbank,aa)

write(0,*)"nmpef_sean 9"
call pefcheck(aa)
! __________________________________________________________
! ___________Call solver for either preconditioned or_______
! ___________just regularized case._________________________
! __________________________________________________________

    if (prec==1) then
!call rmms(databank,"dbank")
       call ilaplacf_init(npatch,lagmark(2)-1,epsl,na)
!      call dot_test(nmhconest_sean_lop,size(pefcoeffs),size(data),dot1,dot2)
!      call dot_test(ilaplacf_lop,size(pefcoeffs),size(pefcoeffs),dot1,dot2)
write(0,*)"PRC"
       call solver_prc(m=pefcoeffs,                               &
                       d=-data,                               &
                      Fop=nmhconest_sean_lop,                      &
                      Sop=ilaplacf_lop,                           &
                      stepper=cgstep,                             &
                      nSop=(product(npatch)*size(aa%hlx(1)%flt)), &
                      niter=niter,                                &
                      eps=eps,                                    &
                      verb=.true.)
       call ilaplacf_close()
    else
       call laplacf_init(npatch,lagmark(2)-1)
!      call dot_test(nmhconest_sean_lop,size(pefcoeffs),size(data),dot1,dot2)
!      call dot_test(laplacf_lop,size(pefcoeffs),size(pefcoeffs),dot1,dot2)
write(0,*)"REG"
       call solver_reg(m=pefcoeffs,                               &
                       d=-data,                               &
                      Fop=nmhconest_sean_lop,                      &
                      Aop=laplacf_lop,                            &
                      stepper=cgstep,                             &
                      nAop=(product(npatch)*size(aa%hlx(1)%flt)), &
                      niter=niter,                                &
                      eps=eps,                                    &
                      verb=.true.)
       call laplacf_close()
    end if
    call nmhconest_sean_close()
    call cgstep_close()
    
! __________________________________________________________
! ___________Re-initialize PEF for original data____________
! __________________________________________________________
!    call set_pef(aa,aa%pch,lagbank,boundbank,pefcoeffs,size(data),lagmark(2)-1)

     do i=1,size(aa%hlx)
	aa%hlx(i)%lag=lagbank(1:size(aa%hlx(i)%lag))
	aa%hlx(i)%flt=pefcoeffs(1+((i-1)*size(aa%hlx(1)%lag)):i*size(aa%hlx(1)%lag))
     end do
call pefcheck(aa)
! __________________________________________________________
! ___________Deallocate bookkeeping devices for scales______
! __________________________________________________________

    deallocate(mask,data,boundbank,lagbank,pefcoeffs)
  end subroutine nmpef_find_sean

end module nmpef_find_sean
