module nmpefe
! Find multi-scale prediction-error filter (helix magic)
  use nmhconest
  use cgstep_mod
  use solver_reg_mod
  use coarsen_norm_new
  use laplacf
  use sep
  use dottest
  use error_check
  implicit none
  contains

  subroutine nmpef_find( yy, aa, nbank, od, oo, a, niter, lbounds, k0, patch, eps)
  ! yy=data aa=filter sc1=scales on,od,oo=datadim a=filtersize niter=#iterations k0=mask

    integer,             intent( in)              :: niter
    integer, dimension(:),pointer                 :: a, cn, patch, oldtable, oldlag
    integer, dimension(:,:)                       :: nbank
    real,    dimension(:)                         :: od, oo
    real, dimension(:), pointer                   :: yy, k0, coarsedata, temp, nd, no,xming,xmaxg,cd,co
    type(nfilter)                                 :: aa
    integer                                       :: is, stat, dim, id, sf, sp, naops
    real                                          :: df, eps, scsize
    real                                          :: oo1d, oo2d, od1d, od2d
    integer, dimension(:), allocatable            :: mark
    logical, dimension(:)                         :: lbounds
    real, dimension(2)                            :: dot1, dot2

  ! set up positions in ee for subsampled data (right side of equation (2) ) 
    dim = size(od)
    allocate (mark(size(nbank,1)+1))
    mark(1) = 1
    do is = 1, size(nbank,1)
      mark(is+1) = mark(is)+product(nbank(is,:))
    end do
write(0,*)"nmpefe 1"
    sf = size(aa%hlx(1)%flt)
    sp = size(aa%hlx)

write(0,*)"nmpefe 2"

    allocate( cn(dim), xming(dim), xmaxg(dim), cd(dim), co(dim) )
    allocate( temp(sf*sp), oldtable(product(nbank(1,:))), oldlag(sf) )

write(0,*)"nmpefe 3"

    oldtable=aa%pch
    oldlag = aa%hlx(1)%lag
    aa%pch=oldtable
    naops=product(patch)*size(aa%hlx(1)%flt)
    temp=0.

write(0,*)"nmpefe 5",count(lbounds), size(lbounds)
    call nmhconest_init( yy, aa, nbank, od, oo, a, lbounds)
write(0,*)"after nmhconest_init"
    deallocate(aa%pch)
    call laplacf_init(patch,size(aa%hlx(1)%flt))

! USE PRECONDITIONED SOLVER

write(0,*)"nmpefe 6"

    call dot_test(nmhconest_lop,size(temp),size(yy),dot1,dot2)
    call dot_test(laplacf_lop,size(temp),size(temp),dot1,dot2)
write(0,*)"nmpefe 7 min/max/size yy",minval(yy),maxval(yy),size(yy)

    call solver_reg( m=temp, d=-yy, Fop=nmhconest_lop, Aop=laplacf_lop, &
stepper=cgstep, nAop=naops, niter=niter, eps=eps, verb=.true.)

    call cgstep_close() 
    call nmhconest_close( )


write(0,*)"nmpefe 8 min/max/size temp",minval(temp),maxval(temp),size(temp)

    allocate(aa%pch(product(nbank(1,:))))
    aa%pch = oldtable
    do is=1,sp
       aa%hlx(is)%lag=oldlag
       aa%hlx(is)%flt = temp((1+(is-1)*sf):(1+(is)*sf))
    end do
    deallocate(mark,temp,oldtable)
  end subroutine 
end module 













