module solver_smp_mod  
! Fit       0 = K W (F m - d)
  use smallchain_mod 
  use  solver_report_mod
  use error_check
  implicit none
  logical, parameter, private  :: T = .true., F = .false.
  contains
  subroutine solver_smp( m,d, Fop, stepper, niter   ,             Wop&
    &,Kop,m0,err,resd,mmov,rmov,verb)
    optional :: Wop,Kop,m0,err,resd,mmov,rmov,verb
    interface
!-------------------------------------------- begin definitions -----------
      integer function Fop(adj,add,m,d)
        real,dimension(:)::m,d
        logical,intent(in)::adj,add
      end function 
      integer function Wop(adj,add,m,d)
        real,dimension(:)::m,d
        logical,intent(in)::adj,add
      end function 
      integer function Kop(adj,add,m,d)
        real,dimension(:)::m,d
        logical,intent(in)::adj,add
      end function 
      integer function stepper(forget,m,g,rr,gg)
        real, dimension(:)  ::        m,g,rr,gg    
        logical             :: forget
      end function
    end interface 
    real, dimension(:),    intent(in)          :: d, m0
    integer,               intent(in)          :: niter
    logical,               intent(in)          :: verb
    real, dimension(:),    intent(out)         :: m,err, resd
    real, dimension(:,:),  intent(out)         ::        rmov, mmov   
    real, dimension(size(m))                   :: g
    real, dimension(size(d)),         target   :: rr, gg
    real, dimension(size(d)+size(m)), target   :: tt
    real, dimension(:), pointer                :: rd, gd, td
    real, dimension(:), pointer                :: rm, gm, tm
    integer                                    :: iter, stat
    logical                                    :: forget
write(0,*)"begin solver smp",size(d),"yeargh!"
    rd => rr(1:size(d))
    gd => gg(1:size(d))
    td => tt(1:size(d))
    tm => tt(1+size(d):)
write(0,*)"solver smp 1"
    if (present( Wop)) then
      stat=Wop(F,F,-d,rd) 
      !--------------- begin initialization -----------
    else
      rd = -d                                                       
      ! Rd  = -W  d
    end if 
!write(0,*)"solver smp 2"
    if (present( m0)) then
      m=m0                                             !  m  =     m0
!write(0,*)"solver smp 3"
      if (present( Wop)) then
!write(0,*)"solver smp 3.1"
        call chain(Wop,Fop,F,T,m,rd,td)
      else
!write(0,*)"solver smp 3.2"
        stat =          Fop(F,T,m,rd   )                
        ! Rd +=  WF m0
      end if
!write(0,*)"solver smp 4"
    else
      m=0
    end if 
!write(0,*)"solver smp 5"
    forget = T
!--------------------------------------------- begin iterations -----------
    do iter = 1,niter  
!write(0,*)"solver_smp loop 1",stat
      if (present(Wop)) then
        call chain(Wop,Fop,T,F,g,rd,td)
      else
        stat =         Fop(T,F,g,rd   )                  
        !  g  = (WF)'Rd
      end if 
!write(0,*)"solver_smp loop 2",stat
      if (present(Kop)) then
        tm=g
        stat = Kop(F,F,tm, g  )
      end if
!write(0,*)"solver_smp loop 3",stat
!  g  =   K  g
      if (present(Wop)) then
        call chain(Wop,Fop,F,F,g,gd,td)
      else
        stat =         Fop(F,F,g,gd   )                  
        ! Gd  = (WF) g
      end if 
!write(0,*)"solver_smp loop 4",stat
!call rmms(m,"   m ")
!call rmms(g,"   g ")
!call rmms(rr,"rr   ")
!call rmms(gg,"gg   ")
      stat = stepper(forget, m,g, rr,gg)   
!write(0,*)"solver_smp loop 5",stat
! m += dm; R += dR
      if (stat .eq.1) then
        exit ! got stuck descending
      end if
!write(0,*)"solver_smp loop 6"
      if (present( mmov)) then
        mmov(:,iter) =  m(:size(mmov,1)) 
        !---------- report -------------
      end if
      if (present( rmov)) then
        rmov(:,iter) = rd(:size(rmov,1))
      end if
      if (present( err )) then
        err(  iter) = dot_product(rd,rd)
      end if
      if (present( verb)) then
        if (verb) then
          call solver_report(iter,m,g,rd)
        end if
      end if
!write(0,*)"solver_smp loop 7"
      forget=F
    end do 
    if (present( resd)) then
      resd = rd
    end if
  end subroutine 
end module 
